@@ -16,8 +16,8 @@ import           Control.Monad.Except           (ExceptT (..), runExceptT,
                                                  throwError)
 import           Control.Monad.Trans            (liftIO)
 import qualified Data.Aeson                     as A
+import qualified Data.Aeson.KeyMap              as AKM
 import           Data.Bifunctor                 (first)
-import qualified Data.HashMap.Strict            as HMS
 import           Data.Maybe                     (fromMaybe)
 import qualified Data.Text                      as T
 import qualified Data.Text.Encoding             as T
@@ -64,7 +64,14 @@ readExtension
     :: ExtensionList -> String
     -> Maybe (T.Text -> Either Pandoc.PandocError Pandoc.Pandoc)
 readExtension (ExtensionList extensions) fileExt = case fileExt of
+    ".markdown"  -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
     ".md"  -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
+    ".mdown"  -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
+    ".mdtext"  -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
+    ".mdtxt"  -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
+    ".mdwn"  -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
+    ".mkd"  -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
+    ".mkdn"  -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
     ".lhs" -> Just $ Pandoc.runPure . Pandoc.readMarkdown lhsOpts
     ""     -> Just $ Pandoc.runPure . Pandoc.readMarkdown readerOpts
     ".org" -> Just $ Pandoc.runPure . Pandoc.readOrg      readerOpts
@@ -121,7 +128,7 @@ readMetaSettings :: T.Text -> Either String PresentationSettings
 readMetaSettings src = case parseMetadataBlock src of
     Nothing -> Right mempty
     Just (Left err) -> Left err
-    Just (Right (A.Object obj)) | Just val <- HMS.lookup "patat" obj ->
+    Just (Right (A.Object obj)) | Just val <- AKM.lookup "patat" obj ->
        resultToEither $! A.fromJSON val
     Just (Right _) -> Right mempty
   where
