@@ -37,20 +37,34 @@ module Patat.PrettyPrint
     , Alignment (..)
     , align
     , paste
+
+    -- * Control codes
+    , removeControls
+    , clearScreen
+    , goToLine
     ) where
 
 
 --------------------------------------------------------------------------------
-import           Control.Monad.Reader (asks, local)
-import           Control.Monad.RWS    (RWS, runRWS)
-import           Control.Monad.State  (get, gets, modify)
-import           Control.Monad.Writer (tell)
-import qualified Data.List            as L
-import           Data.String          (IsString (..))
-import qualified Data.Text            as T
-import           Prelude              hiding (null)
-import qualified System.Console.ANSI  as Ansi
-import qualified System.IO            as IO
+import           Control.Monad.Reader       (asks, local)
+import           Control.Monad.RWS          (RWS, runRWS)
+import           Control.Monad.State        (get, gets, modify)
+import           Control.Monad.Writer       (tell)
+import           Data.Char.WCWidth.Extended (wcstrwidth)
+import qualified Data.List                  as L
+import           Data.String                (IsString (..))
+import qualified Data.Text                  as T
+import           Prelude                    hiding (null)
+import qualified System.Console.ANSI        as Ansi
+import qualified System.IO                  as IO
+
+
+--------------------------------------------------------------------------------
+-- | Control actions for the terminal.
+data Control
+    = ClearScreenControl
+    | GoToLineControl Int
+    deriving (Eq)
 
 
 --------------------------------------------------------------------------------
@@ -58,6 +72,7 @@ import qualified System.IO            as IO
 data Chunk
     = StringChunk [Ansi.SGR] String
     | NewlineChunk
+    | ControlChunk Control
     deriving (Eq)
 
 
@@ -67,17 +82,21 @@ type Chunks = [Chunk]
 
 --------------------------------------------------------------------------------
 hPutChunk :: IO.Handle -> Chunk -> IO ()
-hPutChunk h NewlineChunk            = IO.hPutStrLn h ""
+hPutChunk h NewlineChunk = IO.hPutStrLn h ""
 hPutChunk h (StringChunk codes str) = do
     Ansi.hSetSGR h (reverse codes)
     IO.hPutStr h str
     Ansi.hSetSGR h [Ansi.Reset]
+hPutChunk h (ControlChunk ctrl) = case ctrl of
+    ClearScreenControl -> Ansi.hClearScreen h
+    GoToLineControl l  -> Ansi.hSetCursorPosition h l 0
 
 
 --------------------------------------------------------------------------------
 chunkToString :: Chunk -> String
 chunkToString NewlineChunk        = "\n"
 chunkToString (StringChunk _ str) = str
+chunkToString (ControlChunk _)    = ""
 
 
 --------------------------------------------------------------------------------
@@ -100,7 +119,7 @@ chunkLines chunks = case break (== NewlineChunk) chunks of
 
 
 --------------------------------------------------------------------------------
-data DocE
+data DocE d
     = String String
     | Softspace
     | Hardspace
@@ -108,27 +127,30 @@ data DocE
     | Hardline
     | WrapAt
         { wrapAtCol :: Maybe Int
-        , wrapDoc   :: Doc
+        , wrapDoc   :: d
         }
     | Ansi
         { ansiCode :: [Ansi.SGR] -> [Ansi.SGR]  -- ^ Modifies current codes.
-        , ansiDoc  :: Doc
+        , ansiDoc  :: d
         }
     | Indent
         { indentFirstLine  :: LineBuffer
         , indentOtherLines :: LineBuffer
-        , indentDoc        :: Doc
+        , indentDoc        :: d
         }
+    | Control Control
+    deriving (Functor)
 
 
 --------------------------------------------------------------------------------
-chunkToDocE :: Chunk -> DocE
+chunkToDocE :: Chunk -> DocE Doc
 chunkToDocE NewlineChunk            = Hardline
 chunkToDocE (StringChunk codes str) = Ansi (\_ -> codes) (Doc [String str])
+chunkToDocE (ControlChunk ctrl)     = Control ctrl
 
 
 --------------------------------------------------------------------------------
-newtype Doc = Doc {unDoc :: [DocE]}
+newtype Doc = Doc {unDoc :: [DocE Doc]}
     deriving (Monoid, Semigroup)
 
 
@@ -184,7 +206,7 @@ docToChunks doc0 =
         ((), b, cs) = runRWS (go $ unDoc doc0) env0 mempty in
     optimizeChunks (cs <> bufferToChunks b)
   where
-    go :: [DocE] -> DocM ()
+    go :: [DocE Doc] -> DocM ()
 
     go [] = return ()
 
@@ -228,13 +250,18 @@ docToChunks doc0 =
             go (unDoc indentDoc)
         go docs
 
+    go (Control ctrl : docs) = do
+        tell [ControlChunk ctrl]
+        go docs
+
+
     makeChunk :: String -> DocM Chunk
     makeChunk str = do
         codes <- asks deCodes
         return $ StringChunk codes str
 
     -- Convert 'Softspace' or 'Softline' to 'Hardspace' or 'Hardline'
-    softConversion :: DocE -> [DocE] -> DocM DocE
+    softConversion :: DocE Doc -> [DocE Doc] -> DocM (DocE Doc)
     softConversion soft docs = do
         mbWrapCol <- asks deWrap
         case mbWrapCol of
@@ -242,7 +269,7 @@ docToChunks doc0 =
             Just maxCol -> do
                 -- Slow.
                 currentLine <- gets (concatMap chunkToString . bufferToChunks)
-                let currentCol = length currentLine
+                let currentCol = wcstrwidth currentLine
                 case nextWordLength docs of
                     Nothing                            -> return hard
                     Just l
@@ -254,11 +281,11 @@ docToChunks doc0 =
             Softline  -> Hardline
             _         -> soft
 
-    nextWordLength :: [DocE] -> Maybe Int
+    nextWordLength :: [DocE Doc] -> Maybe Int
     nextWordLength []                 = Nothing
     nextWordLength (String x : xs)
         | L.null x                    = nextWordLength xs
-        | otherwise                   = Just (length x)
+        | otherwise                   = Just (wcstrwidth x)
     nextWordLength (Softspace : xs)   = nextWordLength xs
     nextWordLength (Hardspace : xs)   = nextWordLength xs
     nextWordLength (Softline : xs)    = nextWordLength xs
@@ -266,6 +293,7 @@ docToChunks doc0 =
     nextWordLength (WrapAt {..} : xs) = nextWordLength (unDoc wrapDoc   ++ xs)
     nextWordLength (Ansi   {..} : xs) = nextWordLength (unDoc ansiDoc   ++ xs)
     nextWordLength (Indent {..} : xs) = nextWordLength (unDoc indentDoc ++ xs)
+    nextWordLength (Control _ : _)    = Nothing
 
 
 --------------------------------------------------------------------------------
@@ -278,7 +306,7 @@ toString = concat . map chunkToString . docToChunks
 dimensions :: Doc -> (Int, Int)
 dimensions doc =
     let ls = lines (toString doc) in
-    (length ls, foldr max 0 (map length ls))
+    (length ls, foldr max 0 (map wcstrwidth ls))
 
 
 --------------------------------------------------------------------------------
@@ -297,7 +325,7 @@ putDoc = hPutDoc IO.stdout
 
 
 --------------------------------------------------------------------------------
-mkDoc :: DocE -> Doc
+mkDoc :: DocE Doc -> Doc
 mkDoc e = Doc [e]
 
 
@@ -379,7 +407,7 @@ data Alignment = AlignLeft | AlignCenter | AlignRight deriving (Eq, Ord, Show)
 --------------------------------------------------------------------------------
 align :: Int -> Alignment -> Doc -> Doc
 align width alignment doc0 =
-    let chunks0 = docToChunks doc0
+    let chunks0 = docToChunks $ removeControls doc0
         lines_  = chunkLines chunks0 in
     vcat
         [ Doc (map chunkToDocE (alignLine line))
@@ -387,7 +415,7 @@ align width alignment doc0 =
         ]
   where
     lineWidth :: [Chunk] -> Int
-    lineWidth = sum . map (length . chunkToString)
+    lineWidth = sum . map (wcstrwidth . chunkToString)
 
     alignLine :: [Chunk] -> [Chunk]
     alignLine line =
@@ -406,8 +434,26 @@ align width alignment doc0 =
 -- | Like the unix program 'paste'.
 paste :: [Doc] -> Doc
 paste docs0 =
-    let chunkss = map docToChunks docs0                   :: [Chunks]
-        cols    = map chunkLines chunkss                  :: [[Chunks]]
-        rows0   = L.transpose cols                        :: [[Chunks]]
-        rows1   = map (map (Doc . map chunkToDocE)) rows0 :: [[Doc]] in
+    let chunkss = map (docToChunks . removeControls) docs0 :: [Chunks]
+        cols    = map chunkLines chunkss                   :: [[Chunks]]
+        rows0   = L.transpose cols                         :: [[Chunks]]
+        rows1   = map (map (Doc . map chunkToDocE)) rows0  :: [[Doc]] in
     vcat $ map mconcat rows1
+
+
+--------------------------------------------------------------------------------
+removeControls :: Doc -> Doc
+removeControls = Doc . filter isNotControl . map (fmap removeControls) . unDoc
+  where
+    isNotControl (Control _) = False
+    isNotControl _           = True
+
+
+--------------------------------------------------------------------------------
+clearScreen :: Doc
+clearScreen = mkDoc $ Control ClearScreenControl
+
+
+--------------------------------------------------------------------------------
+goToLine :: Int -> Doc
+goToLine = mkDoc . Control . GoToLineControl
